;; Programs to install less-than-complete graphs for planar subgraph
;; calculation.  Relevant definitions in planarX.lisp.

;; This version of loadgp contains a function to save any graph in 
;; memory.


(in-package '*lisp)

;; Indicator for programs which partition edges, interactions
(defvar partition nil)

;; this saves the up edges as pairs, then -1 -1 as a signal.  Finally,
;; is saves the down edges that are in.  This function is for the graphing
;; function that will use the outfile.
(defun svpresnt (file)
  (with-open-file (*standard-output* file :direction :output)
    (format t "~3D~%" nv)
    (format t "~3D~%"  (b01 legal))
    (*when (and!! edgep prsnt)
         (*if Vup 
              (do-for-selected-processors (e)
                  (format t "~3D ~3D~%" (pref edga e) (pref edgb e))))
         (format t "  -1  -1~%")
         (*if Vdn 
              (do-for-selected-processors (e)
                  (format t "~3D ~3D~%" (pref edga e) (pref edgb e))))
))  ;; end *when, end with-open
)

;; Print current graph from memory to a file that can be reloaded 
;; with f-mk-graph.  

(defun svedges (file)
 (with-open-file (*standard-output* file :direction :output)
    (*when (and!! edgep prsnt)
       (do-for-selected-processors (e)
           (format t "~3D ~3D~%" (pref edga e) (pref edgb e)))
       (format t "-1  -1"))) 
)
  
;; Load graph specified by list of edges present in file (a string).
;; Edges described by a pair of integers: a b, terminated by -1 -1.

(defun ldedges (file)
 (let (a b)
  (*set prsnt nil!!)
  (with-open-file (instrm file)
   (prog ()
    loop
    (setq a (read instrm)) (setq b (read instrm))
    (if (< a 0) (return-from ldedges 'DONE))
    (*when (and!! (=!! edga (!! a)) (=!! edgb (!! b))) (*set prsnt t!!))
    (go loop)
   )))
)

;; Load graph specified by edges not present in file, as for ldedges.

(defun ldmissing (file)
  (ldedges file)
  (*set prsnt (not!! prsnt))
)

;; Load random graph with e edges - complete edges already set up

(defun ldrndm (e)
 (*set prsnt nil!!)
 (*when edgep
   (*set cntx (random!! (!! 1000000)))
   (*set rnvz (rank!! cntx '<=!!))
   (*set prsnt (<!! rnvz (!! e))))
)

;; partition edges

(defun part-edges ()
 (*set cl (self-address-grid!! (!! 0))) ;; Initial class number
 (*set ma cl)  (*set edgc cl)           ;; edgec is temporary self-coordinate
 (*set mb cl)           ;; Right end of class beginning here (initially empty)
 (*when ntrve                                   ;; Find max right end
   (*pset :max edgb mb (cube-from-grid-address!!  edga)))
 (*when (=!! mb ma)             ;; Fix empty classes
    (*set cl (!! -1))  (*set ma cl)  (*set mb (!! (1- nv))))
 (*set fa cl)   (*set fb mb)            ;; Initialize fa, fb
 (*when (<!! edgc (- nv 2))             ;; Select actual representatives
  (if combine
   (dotimes (k (- nv 3))                    ;; Loop to form classes
    ;; (disp-class) (terpri) (terpri)
     (*news ma ma 1)  (*news mb mb 1)           ;; Shift right
     (*when (<=!! mb fa)                        ;; Absorb interval
        (*set mb fb)  (*set cl ma))
    ))  ;; end dotimes, end if
    ;; (disp-class) (terpri) (terpri)
   (*if (=!! cl edgc)           ;; Renumber and count classes
     (progn (*set cl (rank!! cl '<=!!)) (setq Ncl (*sum (!! 1))))
     (*if (plusp!! cl) (*set cl (pref!! cl (cube-from-grid-address!! cl)))))
 )  ;; end *when
 (format t "~D independent edge classes~%" Ncl)
 (*when ntrve                           ;; Get class number
    (*set edgc (pref!! cl (cube-from-grid-address!! edga))))
)

;; Make graph with v vertices, edges defined by file

(defun f-mk-graph (v file)
  (setq nv v)
  (mk-edges)
  (ldedges file)
  (split-edges)
  (if partition (part-edges))
  (mk-inter)
)

;; Make random graph with v vertices, e edges chosen from complete graph,
;; whose edges should fit in default-vp-set.

(defun cr-mk-graph (v e)
  (setq nv v)
  (mk-edges)
  (ldrndm e)
  (split-edges)
  (if partition (part-edges))
  (mk-inter)
)

;; Make random graph with v vertices, e edges, e <= size of default-vp-set.
;; May produce fewer edges unless e appreciably less than DSZ.

(defun er-mk-graph (v e)
  (setq nv v)
  (mk-random-edges e)
  (split-edges)
  (if partition (part-edges))
  (mk-inter)
)

;; Create e random edges (perhaps fewer) - nothing yet installed

(defun mk-random-edges (e)
;; Clear important parts of existing representation:
  (*set edgep nil!!)  (*set edgsg nil!!)  (*set ntrve nil!!)
  (*set prsnt nil!!)
  (setq outcome 'SUCCESS)
;; Make lots of random edges:
  (*set edga (random!! (!! nv)))  (*set edgb (random!! (!! (1- nv))))
  ;; Force distinct vertices:
  (*when (<=!! edga edgb) (*set edgb (1+!! edgb)))
  ;; Order vertices:
  (*when (>!! edga edgb) (*set cntx edga) (*set edga edgb) (*set edgb cntx))
;; Sort edges and eliminate duplicates:
  (*let ((eab (logior!! (ash!! edga (!! NS)) edgb)) (rab (!! 0)))
        (declare (type (field-pvar (* 2 NS)) eab rab))
    (*set rnvz (rank!! eab '<=!!))
    (*pset :no-collisions eab rab rnvz)
    (*when (and!! (not!! (zerop!! rnvz)) (>!! eab (pref!! rab (1-!! rnvz))))
       ;; Active processors represent distinct edges
       (setq ce (*sum (!! 1)))
       (*pset :no-collisions edga edga rnvz :notify edgep)
       (*pset :no-collisions edgb edgb rnvz)
    )) ; end *when and!!, end *let
;; Select final edges and finish representation:
  (ldrndm e)
  (setq ce (*sum (!! 1)))
  (*set ntrve (not!! (or!! (=!! edgb (1+!! edga))   ; Non-trivial edges
                           (and!! (zerop!! edga) (=!! edgb (!! (1- nv)))))))
)
